"""
Author: C. Nichols
Created: Feb. 06, 2012

ICON By http://www.famfamfam.com/

COMPILE -
Only compiles correctly if you compile from the IronPython directory...
> ipy.exe Tools\Scripts\pyc.py /out:FinalCountDown /main:FinalCountDown.py /target:winexe

"""
import sys
import os
import stat
import time
import datetime

# =============================================================================
# Get the latest .Net framework and add to path.
winnet_basepath=r"C:\Windows\Microsoft.NET\Framework"
net_libs=[]
if os.path.exists(winnet_basepath):
    for subpath in os.listdir(winnet_basepath):
        full_path = os.path.join(winnet_basepath,subpath)
        if os.path.isfile(full_path): continue
        #sys.path.append(full_path) # Add them all...
        net_libs.append( ( os.stat(full_path)[stat.ST_CTIME], full_path ) )
    net_libs.sort()
    sys.path.append(net_libs[-1][-1])
else:
    print ".Net Framework not found at %s!" %winnet_basepath
# =============================================================================
import __main__

# Make clr accessable.
import clr
import System

# Add the forms ref.
clr.AddReference('System.Windows.Forms')
clr.AddReference('System.Drawing')

# Import .Net assemblies.
from System.Drawing import Icon
from System.Windows.Forms import (Form, Application, FormWindowState,
                                  MouseButtons, ContextMenu, MenuItem,
                                  NotifyIcon, MessageBox, Timer)

# =============================================================================
# GLOBALS:
# =============================================================================
INPUT_DATE = ''
scriptDirectory = os.path.dirname(__main__.__file__)
ConfigFilename = os.path.join(scriptDirectory, 'finalcountdown.cfg')
IconFilename = os.path.join(scriptDirectory, 'hourglass.ico')

# If no config exists create one.
confFile = "# Set end date:\n# <yyyymmdd>\n12312018"
if not os.path.exists(ConfigFilename):
    fh=open(ConfigFilename,'w')
    fh.write(confFile.strip())
    fh.close()
    
# Read the set date from config.
for line in open(ConfigFilename,'r'):
    line=line.strip()
    if not line: continue
    if line.startswith('#'): continue
    INPUT_DATE = line.strip()

def saveSetting(arg):
    # Save the user's date entry.
    global INPUT_DATE
    fh=open(ConfigFilename,'w')
    fh.write(arg.strip())
    fh.close()

    INPUT_DATE = arg.strip() 
# =============================================================================
# Form Classes (Windows and stuff)
# =============================================================================
class MyDialog(System.Windows.Forms.Form):
    def __init__(self):
        # I create a new Window.
        icon = Icon(IconFilename)
        
        # Config the form.
        self.ClientSize = System.Drawing.Size(220,110)
        self.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog
        self.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen
        self.MaximizeBox = 0
        self.MinimizeBox = 0
        
        self.Text = "Settings"
        self.Icon = icon

        self.lblInput = System.Windows.Forms.Label()
        self.lblInput.Text = "Enter End Date (mmddyyyy):"
        self.lblInput.Location = System.Drawing.Point(10,15)
        self.lblInput.Size = System.Drawing.Size(215,30)     

        self.txtInput = System.Windows.Forms.TextBox()
        self.txtInput.Text = INPUT_DATE
        self.txtInput.Location = System.Drawing.Point(15, 45)
        self.txtInput.Width = 150
        self.txtInput.Height = 50

        self.btnSave = System.Windows.Forms.Button()
        self.btnSave.Text = "&Save"
        self.btnSave.Location = System.Drawing.Point(135,80)
        self.btnSave.Click += self.DialogSave

        self.Controls.Add(self.lblInput)
        self.Controls.Add(self.txtInput) 
        self.Controls.Add(self.btnSave)  

    def DialogSave(self, sender, event):
        if self.txtInput.Text.isdigit():
            saveSetting(self.txtInput.Text)
            self.DialogResult = System.Windows.Forms.DialogResult.OK
            self.Close()
        else:
            MessageBox.Show("You need to enter a date!", "Stop that!")

    
class Main(System.Windows.Forms.Form):

    def __init__(self):
        self.initNotifyIcon()
        self.initWindow()
        timer = Timer()
        timer.Interval = 3600000
        timer.Tick += self._upDate
        timer.Start()
        
    def initWindow(self):     
        # Embed Icon File: Needs to be in the root with the exe...
        icon = Icon(IconFilename)
        
        font = System.Drawing.Font("Times Roman", 11)

        # Config the form.
        self.ClientSize = System.Drawing.Size(220,110)
        
        self.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog
        self.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen
        self.MaximizeBox = 0
        self.MinimizeBox = 0
        
        self.Text = "Final CountDown!"
        self.Icon = icon

        self.lblLabel = System.Windows.Forms.Label()
        self.lblLabel.AutoSize = True
        self.lblLabel.Font = font
        self.lblLabel.Text = "Current Count:"
        self.lblLabel.Location = System.Drawing.Point(20,15)        

        self.lblDates = System.Windows.Forms.Label()
        self.lblDates.AutoSize = True
        self.lblDates.Font = font
        self.lblDates.Text = self._upDate(None,None) # init label
        self.lblDates.Location = System.Drawing.Point(40,40)

        self.btnSetDate = System.Windows.Forms.Button()
        self.btnSetDate.Text = "&Set Date"
        self.btnSetDate.Location = System.Drawing.Point(10,80)
        self.btnSetDate.Click += self.SetDate

        self.btnClose = System.Windows.Forms.Button()
        self.btnClose.Text = "&Close"
        self.btnClose.Location = System.Drawing.Point(135,80)
        self.btnClose.Click += self.ShowClose        

        self.Controls.Add(self.lblDates)
        self.Controls.Add(self.lblLabel) 
        self.Controls.Add(self.btnClose)
        self.Controls.Add(self.btnSetDate)        
    
    def initNotifyIcon(self):
        self.notifyIcon = NotifyIcon()
        self.notifyIcon.Icon = Icon(IconFilename)
        self.notifyIcon.Visible = False

        self.notifyIcon.MouseDoubleClick += self.DoubleClickOnTrayIcon        
        self.notifyIcon.ContextMenu = self.initContextMenu()

        initValue = self._upDate(None,None)
        self.notifyIcon.Text = initValue      

    def _upDate(self, sender, event):
        # Get current date
        now = datetime.datetime.now()

        # Get user's date        
        secs=time.mktime(time.strptime(INPUT_DATE.strip(),'%m%d%Y'))
        deadline = datetime.datetime.fromtimestamp(secs)

        diffa = deadline-now 

        strOut = "%s days remaining" %str(abs(diffa.days))
        strTipOut = str(abs(diffa.days))
        
        # Init first display...
        if not sender: return strOut # init default label.

        # Update         
        self.lblDates.Text = strOut     # Update form label
        self.notifyIcon.Visible = True  
        self.notifyIcon.Text = strOut   # Update systray tip

        #self.notifyIcon.BalloonTipText = strTipOut
        #self.notifyIcon.ShowBalloonTip(3000)

        if self.Visible:
            self.notifyIcon.Visible = False
        else:
            self.notifyIcon.Visible = True

    def initContextMenu(self):
        # Systray Menu Items
        contextMenu = ContextMenu()
        
        setShowMenuItem = MenuItem("Show Counter")
        setShowMenuItem.Click += self.Show
        contextMenu.MenuItems.Add(setShowMenuItem)
        
        exitMenuItem = MenuItem("Exit")
        exitMenuItem.Click += self.Exit
        contextMenu.MenuItems.Add(exitMenuItem)
        
        return contextMenu
    
    # OUR BUTTON ACTIONS
    def Show(self, sender, event):
        #if self.WindowState == FormWindowState.Minimized:
        self.notifyIcon.Visible = False
        self.Visible = True
        
    def ShowClose(self, sender, event):
        self.notifyIcon.Visible = True
        self.Visible = False

    def Exit(self, sender, event):
        self.notifyIcon.Visible = False
        Application.Exit()

    def DoubleClickOnTrayIcon(self, s, e):
        if e.Button == MouseButtons.Left:
            self.Visible = True
            if self.WindowState == FormWindowState.Minimized:
                self.WindowState = FormWindowState.Normal
            self.notifyIcon.Visible = False
            
    def SetDate(self, sender, event):
        dialog = MyDialog()
        dialog.ShowDialog()
        dialog.Dispose()
        
        #Update labels in main view on save.
        updated_text = self._upDate(None,None)
        self.lblDates.Text = updated_text
        self.notifyIcon.Visible = False  
        self.notifyIcon.Text = updated_text
        self.notifyIcon.BalloonTipText = updated_text
        self.notifyIcon.ShowBalloonTip(2000)   
        
    def Run(self):
        System.Windows.Forms.Application.Run(self)      
        
if __name__ == "__main__":
    # Run
    main = Main()
    Application.EnableVisualStyles()
    main.Run()
